import 'package:flutter/foundation.dart';
import 'package:flutter_test/flutter_test.dart';

import 'apple_pay_api.dart';
import 'audio_playback_policy.dart';
import 'clear_cache.dart';
import 'clear_client_cert_preferences.dart';
import 'clear_focus.dart';
import 'clear_ssl_preferences.dart';
import 'content_blocker.dart';
import 'create_pdf.dart';
import 'get_certificate.dart';
import 'get_content_height.dart';
import 'get_current_web_view_package.dart';
import 'get_default_user_agent.dart';
import 'get_favicons.dart';
import 'get_html.dart';
import 'get_meta_tags.dart';
import 'get_meta_theme_color.dart';
import 'get_original_url.dart';
import 'get_progress.dart';
import 'get_title.dart';
import 'handles_url_scheme.dart';
import 'http_auth_credential_database.dart';
import 'initial_url_request.dart';
import 'inject_css.dart';
import 'inject_javascript_file.dart';
import 'intercept_ajax_request.dart';
import 'intercept_fetch_request.dart';
import 'is_loading.dart';
import 'is_secure_context.dart';
import 'javascript_code_evaluation.dart';
import 'javascript_dialogs.dart';
import 'javascript_handler.dart';
import 'load_data.dart';
import 'load_file.dart';
import 'load_file_url.dart';
import 'load_url.dart';
import 'on_console_message.dart';
import 'on_download_start_request.dart';
import 'on_js_before_unload.dart';
import 'on_received_error.dart';
import 'on_received_http_error.dart';
import 'on_load_resource.dart';
import 'on_load_resource_with_custom_scheme.dart';
import 'on_navigation_response.dart';
import 'on_page_commit_visible.dart';
import 'on_permission_request.dart';
import 'on_print.dart';
import 'on_progress_changed.dart';
import 'on_received_icon.dart';
import 'on_received_touch_icon_url.dart';
import 'safe_browsing.dart';
import 'on_scroll_changed.dart';
import 'on_title_changed.dart';
import 'on_update_visited_history.dart';
import 'on_window_blur.dart';
import 'on_window_focus.dart';
import 'page_down_up.dart';
import 'pause_resume.dart';
import 'programmatic_zoom_scale.dart';
import 'pause_resume_timers.dart';
import 'post_requests.dart';
import 'print_current_page.dart';
import 'programmatic_scroll.dart';
import 'pull_to_refresh.dart';
import 'reload.dart';
import 'request_focus_node_href.dart';
import 'request_image_ref.dart';
import 'resize_webview.dart';
import 'web_archive.dart';
import 'set_custom_useragent.dart';
import 'set_get_settings.dart';
import 'set_web_contents_debugging_enabled.dart';
import 'should_intercept_request.dart';
import 'should_override_url_loading.dart';
import 'ssl_request.dart';
import 'stop_loading.dart';
import 't_rex_runner_game.dart';
import 'take_screenshot.dart';
import 'user_scripts.dart';
import 'video_playback_policy.dart';
import 'web_history.dart';
import 'web_message.dart';
import 'webview_asset_loader.dart';
import 'webview_windows.dart';

void main() {
  final shouldSkip = [TargetPlatform.macOS].contains(defaultTargetPlatform);

  group('InAppWebView', () {
    initialUrlRequest();
    setGetSettings();
    javascriptCodeEvaluation();
    loadUrl();
    loadFileUrl();
    javascriptHandler();
    resizeWebView();
    setCustomUserAgent();
    videoPlaybackPolicy();
    audioPlaybackPolicy();
    getTitle();
    programmaticScroll();
    shouldOverrideUrlLoading();
    onReceivedError();
    webViewWindows();
    interceptAjaxRequest();
    interceptFetchRequest();
    contentBlocker();
    httpAuthCredentialDatabase();
    onConsoleMessage();
    onDownloadStartRequest();
    javascriptDialogs();
    onReceivedHttpError();
    onLoadResourceWithCustomScheme();
    onLoadResource();
    onUpdateVisitedHistory();
    onProgressChanged();
    safeBrowsing();
    onScrollChanged();
    sslRequest();
    onPrint();
    onWindowFocus();
    onWindowBlur();
    onPageCommitVisible();
    onTitleChanged();
    programmaticZoomScale();
    onPermissionRequest();
    shouldInterceptRequest();
    onReceivedIcon();
    onReceivedTouchIconUrl();
    onJsBeforeUnload();
    onNavigationResponse();
    postRequests();
    loadData();
    loadFile();
    reload();
    webHistory();
    getProgress();
    getHtml();
    getFavicons();
    isLoading();
    stopLoading();
    injectJavascriptFile();
    injectCSS();
    takeScreenshot();
    clearCache();
    tRexRunnerGame();
    pauseResumeTimers();
    printCurrentPage();
    getContentHeight();
    clearFocus();
    requestFocusNodeHref();
    requestImageRef();
    getMetaTags();
    getMetaThemeColor();
    getCertificate();
    userScripts();
    webArchive();
    isSecureContext();
    getDefaultUserAgent();
    pullToRefresh();
    webMessage();
    clearSslPreferences();
    pauseResume();
    getOriginalUrl();
    pageDownUp();
    clearClientCertPreferences();
    getCurrentWebViewPackage();
    setWebContentsDebuggingEnabled();
    createPdf();
    applePayAPI();
    handlesURLScheme();
    webViewAssetLoader();
  }, skip: shouldSkip);
}
